/* BEGIN software license
 *
 * MsXpertSuite - mass spectrometry software suite
 * -----------------------------------------------
 * Copyright(C) 2009,...,2018 Filippo Rusconi
 *
 * http://www.msxpertsuite.org
 *
 * This file is part of the MsXpertSuite project.
 *
 * The MsXpertSuite project is the successor of the massXpert project. This
 * project now includes various independent modules:
 *
 * - massXpert, model polymer chemistries and simulate mass spectrometric data;
 * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * END software license
 */

#pragma once


/////////////////////// Local includes
#include "exportimportconfig.h"
#include "PolChemDefEntity.hpp"
#include "Formula.hpp"
#include "Modif.hpp"


namespace MsXpS
{


namespace libXpertMass
{


class DECLSPEC CrossLinker : public PolChemDefEntity, public Formula, public Ponderable
{
  public:
  CrossLinker(PolChemDefCstSPtr, const QString &, const QString &);

  CrossLinker(const CrossLinker &);

  ~CrossLinker();

  bool setModifAt(Modif *, int);
  bool appendModif(Modif *);
  const Modif *modifAt(int) const;
  bool removeModifAt(int);

  QString formula() const;

  QList<Modif *> &modifList();

  int hasModif(const QString &);

  using PolChemDefEntity::operator=;
  using Formula::operator=;
  using Ponderable::operator=;
  CrossLinker &operator=(const CrossLinker &other);

  using PolChemDefEntity::operator==;
  using Formula::operator==;
  using Ponderable::operator==;
  virtual bool operator==(const CrossLinker &) const;

  int isNameKnown();
  static int isNameInList(const QString &,
                          const QList<CrossLinker *> &,
                          CrossLinker * = 0);

  using Formula::validate;
  using PolChemDefEntity::validate;
  bool validate();

  using Formula::accountMasses;
  using Ponderable::accountMasses;
  virtual bool accountMasses(double * = 0, double * = 0, int = 1);
  virtual bool accountMasses(Ponderable *, int = 1);

  bool calculateMasses();

  bool renderXmlClkElement(const QDomElement &, int);

  QString *formatXmlClkElement(int, const QString & = QString("  "));

  protected:
  // We do not own the modifications below, the pointers refer to
  // modifications in the polymer chemistry definition.
  QList<Modif *> m_modifList;
};

} // namespace libXpertMass

} // namespace MsXpS


