/* BEGIN software license
 *
 * MsXpertSuite - mass spectrometry software suite
 * -----------------------------------------------
 * Copyright (C) 2009--2020 Filippo Rusconi
 *
 * http://www.msxpertsuite.org
 *
 * This file is part of the MsXpertSuite project.
 *
 * The MsXpertSuite project is the successor of the massXpert project. This
 * project now includes various independent modules:
 *
 * - massXpert, model polymer chemistries and simulate mass spectrometric data;
 * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * *****************************************************************************
 * This specific code is a port to C++ of the Envemind Python code by Radzinski
 * and colleagues of IsoSpec fame (Lacki, Startek and company :-)
 *
 * See https://github.com/PiotrRadzinski/envemind.
 * *****************************************************************************
 *
 * END software license
 */


#pragma once

/////////////////////// Qt includes
#include <QDebug>
#include <QTcpSocket>


/////////////////////// Local includes
#include "exportimportconfig.h"


namespace MsXpS
{
namespace libXpertMass
{

class DECLSPEC MassDataClient : public QObject
{
  Q_OBJECT

  public:
  explicit MassDataClient(const QString &ip_address,
                          int port_number,
                          int connection_frequency = 1,
                          QObject *parent = nullptr);
  virtual ~MassDataClient();
  QString getStatus();

  protected slots:
  void reportError(QAbstractSocket::SocketError socket_error);
  void requestNewData();
  virtual void readData();

  signals:
  void reportErrorSignal(const QString &error);
  void newDataSignal(const QByteArray &byte_array);
  void hostFoundSignal();
  void connectedSignal();

  protected:
  QDataStream m_inStream;
  QByteArray m_data;
  QString m_ipAddress;
  int m_portNumber;
  int m_connectionFrequency = 1;
  QTcpSocket *mpa_tcpSocket = nullptr;
};

} // namespace libXpertMass

} // namespace MsXpS
